"""
plane_env.py
An environment with a complex plane for agent's to move in.
"""

import math
import cmath
import random
import logging
import indra.spatial_env as se

REAL = 0
IMAG = 1


def pos_to_complex(pos):
    return complex(pos[REAL], pos[IMAG])


def rand_complex(init_pos, radius):
    """
    Generates a random complex number
    uniformly picked from a disk of radius 'radius'.
    """
    init = pos_to_complex(init_pos)
    radius = math.sqrt(radius * random.uniform(0.0, radius))
    theta = random.random() * 2 * math.pi
    return init + cmath.rect(radius, theta)


def pos_msg(agent, pos):
    """
    A convenience function for displaying
    an entity's position.
    """
    return("New position for " +
           agent.name + " is "
           + pos_to_str(pos))


def pos_to_str(pos):
    """
    Convert a complex number to a string rep.
    """
    return str(int(pos.real)) + " , " + str(int(pos.imag))


class PlaneEnv(se.SpatialEnv):
    """
    Extends SpatialEnv with entities located in the
    complex plane.
    """
    def __init__(self, name, width, height, preact=True,
                 postact=False, model_nm=None):

        super().__init__(name, width, height, preact=preact,
                         postact=postact, model_nm=model_nm)

    def preact_loop(self):
        """
        Before acting, get agent's new location.
        If we again establish plane agents, this must be written anew.
        """
        logging.debug("Calling preact_loop()")
        for agent in self.agents:
            pass

    def add_agent(self, agent):
        """
        Add a spatial agent to env
        """
        super().add_agent(agent)
        self.position_item(agent)

        v = agent.get_type()
        logging.debug("Adding " + agent.__str__()
                      + " of variety " + v)

    def position_item(self, agent):
        x = random.uniform(0, self.width - 1)
        y = random.uniform(0, self.height - 1)
        agent.pos = complex(x, y)

    def closest_x(self, seeker, prehensions):
        """
        What is the closest entity of target_type?
        """
        pos = pos_to_complex(seeker.pos)
        x = self.max_dist
        closest_target = None
        logging.debug("About to search for closest, among "
                      + str(len(prehensions)))
        for agent in prehensions:
            if seeker is not agent:  # don't locate me!
                if(seeker.exclude is None
                   or (agent not in seeker.exclude)):
                    p_pos = pos_to_complex(agent.pos)
                    d = abs(pos - p_pos)
                    if d < x:
                        x = d
                        closest_target = agent
        logging.debug("Going to return closest = " + str(closest_target))
        return closest_target

    def get_new_wander_pos(self, agent):
        """
        If an agent is wandering in the env,
        this assigns it a new, random position
        based on its current position
        This is not used at present.
        """
        new_pos = rand_complex(agent.pos, agent.max_move)
        x = new_pos.real
        y = new_pos.imag
        if x < 0.0:
            x = 0.0
        if y < 0.0:
            y = 0.0
        if x > self.width:
            x = self.width
        if y > self.height:
            y = self.height
        pos = complex(x, y)
        logging.debug(pos_msg(agent, pos))
        return pos



#!/Users/gcallah/miniconda3/bin/python
"""
This is a simple test script. It can be cloned to
create new run scripts, and should be run to test
the system after library changes.
"""

import indra.prop_args as props
import indra.utils as utils
import basic_model as bm

# set up some file names:
MODEL_NM = "basic_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a
# "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_agent_num(pa, "num_agents", "agents", 16)

# Now we create a minimal environment for our agents to act within:
env = bm.BasicEnv(model_nm=MODEL_NM)

# Now we loop creating multiple agents
#  with numbered names based on the loop variable:
for i in range(pa.get("num_agents")):
    env.add_agent(bm.BasicAgent(name="agent" + str(i),
                                goal="acting up!"))

utils.run_model(env, prog_file, results_file)



import indra.grid_agent as ga

X = 0
Y = 1


class Bacterium(ga.GridAgent):
    """
    An agent that randomly jumps to an empty cell every turn. does not yet seek
    the foodsource
    """

    def act(self):
        (x, y) = self.pos
        print("My name is " + self.name + " and I am a " + self.ntype
              + ".")
        print("My position is" + " x = " + str(x) + ","
              + " y = " + str(y))
        for neighbor in self.neighbor_iter():
            """(x1, y1) = neighbor.pos
            print("    %i, %i" % (x1, y1))"""

    def postact(self):
        self.env.move_to_empty(self)


class FoodSource(ga.GridAgent):
    """
    An agent that checks its neighbors every turn. If one of those neighbors is
    a Bacterium, it will run away to a random location
    """

    def act(self):
        (x, y) = self.pos
        print("I am a " + self.ntype)
        print("My position is x = " + str(x) + ", y = " + str(y))
        for neighbor in self.neighbor_iter():
            if neighbor.ntype == "Bacterium":
                print("Enemy near! Run away!")
                self.env.move_to_empty(self)
                (x, y) = self.pos
                print("My new position is x = " + str(x) + ", y = " + str(y))
            else:
                print("nothing here")

"""
    def postact(self):
        self.env.move_to_empty(self)
"""



from indra.entry_point import entry_point

models = {
             "1": ("Adam Smith's Fashion Model", "fashion_run.py"),
             "2": ("Forest Fire", "forestfire_run.py"),
             "3": ("Abelian Sandpile", "sand_run.py"),
             "4": ("Wolf Sheep", "wolfsheep_run.py")
         }


def main(args=None):
    entry_point(args, models)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
A predator-prey model with wolves and sheep.
"""

import indra.utils as utils
import indra.prop_args as props
import wolfsheep_model as wsm

# set up some file names:
MODEL_NM = "wolfsheep_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_grid_dims(pa, 16)
    utils.get_agent_num(pa, "num_wolves", "wolves", 5)
    utils.get_agent_num(pa, "num_sheep", "sheep", 16)
    pa.ask("wolf_repro", "What is the wolf reproduction age?", int, default=16)
    pa.ask("wolf_lforce", "What is the wolf life force?", int, default=7)
    pa.ask("sheep_repro", "What is the sheep reproduction age?", int,
           default=3)
    pa.ask("sheep_lforce", "What is the sheep life force?", int, default=6)

# Now we create a meadow for our agents to act within:
env = wsm.Meadow("Meadow",
                 pa.get("grid_width"),
                 pa.get("grid_height"),
                 model_nm=MODEL_NM,
                 preact=True,
                 postact=True)

# Now we loop creating multiple agents with numbered names
# based on the number of agents of that type to create:
for i in range(pa.get("num_wolves")):
    env.add_agent(wsm.Wolf("wolf" + str(i), "Eating sheep",
                           pa.get("wolf_repro"),
                           pa.get("wolf_lforce"),
                           rand_age=True))
for i in range(pa.get("num_sheep")):
    env.add_agent(wsm.Sheep("sheep" + str(i), "Reproducing",
                            pa.get("sheep_repro"),
                            pa.get("sheep_lforce"),
                            rand_age=True))

utils.run_model(env, prog_file, results_file)



#!/usr/bin/env python3
"""
Runs a fashion model with hipsters and followers.
"""

import indra.utils as utils
import indra.prop_args as props
import fashion_model as fm

# set up some file names:
MODEL_NM = "fashion_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_grid_dims(pa, 16)
    utils.get_agent_num(pa, "num_followers", "followers", 48)
    utils.get_agent_num(pa, "num_hipsters", "hipsters", 16)
    utils.get_max_move(pa, "fmax_move", "follower", 4)
    utils.get_max_move(pa, "hmax_move", "hipster", 4)
    utils.get_pct(pa, "variability", "agent", "variability", .15)

# Now we create a minimal environment for our agents to act within:
env = fm.Society("Society",
                 pa.get("grid_height"),
                 pa.get("grid_width"),
                 torus=False,
                 model_nm=MODEL_NM)

# Now we loop creating multiple agents with numbered names
# based on the loop variable:
for i in range(pa.get("num_followers")):
    env.add_agent(fm.Follower("follower" + str(i), "Looking like hipsters",
                              pa.get("fmax_move"), pa.get("variability")))
for i in range(pa.get("num_hipsters")):
    env.add_agent(fm.Hipster("hipster" + str(i), "Looking trendy",
                             pa.get("hmax_move"), pa.get("variability")))

utils.run_model(env, prog_file, results_file)



#!/usr/bin/env python3
"""
This file runs the forestfire_model.
"""

import indra.prop_args as props
import indra.utils as utils
import forestfire_model as fm

DEF_GRID_DIM = 80
DEF_DENS = .43
DEF_REGEN = 20
DEF_LIGHTNING = 4

# set up some file names:
MODEL_NM = "forestfire_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a
# "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_grid_dims(pa, DEF_GRID_DIM)
    utils.get_pct(pa, "density", "forest", "density", DEF_DENS)
    pa.ask("strike_freq", "How many turns between lightning strikes?",
           int, default=DEF_LIGHTNING, limits=utils.POS_INTS)
    pa.ask("regen_period",
           "How many turns before a new tree grows where one has burned?",
           int, default=DEF_REGEN, limits=utils.POS_INTS)

density = pa.get("density")
grid_x = pa.get("grid_width")
grid_y = pa.get("grid_height")

# Now we create a forest environment for our agents to act within:
env = fm.ForestEnv(grid_x, grid_y, density, pa.get("strike_freq"),
                   pa.get("regen_period"), model_nm=MODEL_NM, torus=False)
num_agents = int(grid_x * grid_y * density)

for i in range(num_agents):
    env.add_agent(fm.Tree(name="tree" + str(i)))

utils.run_model(env, prog_file, results_file)



#!/usr/bin/env python3
"""
This is a simple test script. It can be cloned to
create new run scripts, and should be run to test
the system after library changes.
"""

import time
import logging
import indra.utils as utils
import indra.prop_args as props
import timer_model as tm

# set up some file names:
MODEL_NM = "timer_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a
# "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    pa.set("num_agents", 50)

# Now we create a minimal environment for our agents to act within:
env = tm.TimerEnv(model_nm=MODEL_NM)

# Now we loop creating multiple agents
#  with numbered names based on the loop variable:
for i in range(pa.get("num_agents")):
    env.add_agent(tm.TimerAgent1(name="agent" + str(i),
                                 goal="acting up!"))
    env.add_agent(tm.TimerAgent2(name="agent" + str(i),
                                 goal="acting up!"))
    env.add_agent(tm.TimerAgent3(name="agent" + str(i),
                                 goal="acting up!"))

# Logging is automatically set up for the modeler:
logging.info("Starting program " + prog_file)

# And now we set things running!
print(time.clock())
env.run(loops=10000)
print(time.clock())



#!/usr/bin/env python3
"""
A script to test our grid capabilities.
"""

import indra.utils as utils
import indra.prop_args as props
import sand_model as sm

# set up some file names:
MODEL_NM = "grid_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_grid_dims(pa, 200)

# Now we create a minimal environment for our agents to act within:
env = sm.SandEnv("Abelian sand env",
                 pa.get("grid_width"),
                 pa.get("grid_height"),
                 model_nm=MODEL_NM)

# This env adds agents itself.

utils.run_model(env, prog_file, results_file)



"""
Filename: plane_agent.py
Author: Gene Callahan
"""

# import logging
import indra.spatial_agent as sa
import indra.plane_env as pe

MAX_EXCLUDE = 10


class PlaneAgent(sa.SpatialAgent):
    """
    This class is the parent of all entities that are
    located on a plane env.
    """

    def __init__(self, name, goal, max_move=0.0, max_detect=0.0):
        super().__init__(name, goal, max_move, max_detect)

    @property
    def pos(self):
        return (self.__pos.real, self.__pos.imag)

    @pos.setter
    def pos(self, pos):
        self.__pos = pos

    def in_range(self, prey, dist):
        """
        Is one agent in range of another in some sense?
        """
        if prey is None:
            return False
        my_pos = pe.pos_to_complex(self.pos)
        prey_pos = pe.pos_to_complex(prey.pos)
        if abs(my_pos - prey_pos) < dist:
            return True
        else:
            return False

    def detect_behavior(self):
        """
        What to do on detecting a prehension.
        """
        pass



#!/usr/bin/env python3
"""
A script to test our two pop capabilities.
"""

import indra.utils as utils
import indra.prop_args as props
import indra.two_pop_model as itpm
import two_pop_model as tpm

# set up some file names:
MODEL_NM = "two_pop_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_grid_dims(pa, 6)
    utils.get_agent_num(pa, "num_agents", "agents", 8)

# Now we create a minimal environment for our agents to act within:
env = itpm.TwoPopEnv("Test grid env",
                 pa.get("grid_width"),
                 pa.get("grid_height"),
                 torus=False,
                 model_nm=MODEL_NM,
                 postact=True)

# Now we loop creating multiple agents with numbered names
# based on the loop variable:

# Get Number of Followers
for i in range(pa.get("num_agents")):
    env.add_agent(tpm.TestFollower(name="Follower" + str(i),
                  goal="taking up a grid space!",max_move=1))

# Get Number of Hipsters
for i in range(pa.get("num_agents")):
    env.add_agent(tpm.TestLeader(name="Leader" + str(i),
                  goal="taking up a grid space!",max_move=1))

utils.run_model(env, prog_file, results_file)



#!/usr/bin/env python3
"""
Runs a financial market model with value investors and chart followers.
"""

import indra.utils as utils
import indra.prop_args as props
import fmarket_model as fm

# set up some file names:
MODEL_NM = "fmarket_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_grid_dims(pa, 16)
    utils.get_agent_num(pa, "num_followers", "followers", 32)
    utils.get_agent_num(pa, "num_vinvestors", "value investors", 16)
    utils.get_max_move(pa, "fmax_move", "chart follower", 4)
    utils.get_max_move(pa, "vmax_move", "value investor", 4)
    utils.get_pct(pa, "variability", "agent", "variability", .2)

# Now we create a asset environment for our agents to act within:
env = fm.FinMarket("Financial Market",
                   pa.get("grid_height"),
                   pa.get("grid_width"),
                   torus=False,
                   model_nm=MODEL_NM)

# Now we loop creating multiple agents with numbered names
# based on the loop variable:
for i in range(pa.get("num_followers")):
    env.add_agent(fm.ChartFollower("follower" + str(i), "Following trend",
                                   pa.get("fmax_move"), pa.get("variability")))
for i in range(pa.get("num_vinvestors")):
    env.add_agent(fm.ValueInvestor("value_inv" + str(i), "Buying value",
                                   pa.get("vmax_move"), pa.get("variability")))

utils.run_model(env, prog_file, results_file)



#!/usr/bin/env python3
"""
A script to run the bacterium model.
"""

import indra.utils as utils
import indra.prop_args as props
import indra.grid_env as ge
import bacterium_model as bm

# set up some file names:
MODEL_NM = "grid_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    pa.set("num_agents", 1)
    pa.set("grid_width", 4)
    pa.set("grid_height", 4)

# Now we create a minimal environment for our agents to act within:
env = ge.GridEnv("Test grid env",
                 pa.get("grid_height"),
                 pa.get("grid_width"),
                 torus=False,
                 model_nm=MODEL_NM,
                 postact=True)

# Now we loop creating multiple agents with numbered names
# based on the loop variable:
"""
for i in range(pa.get("num_agents")):
"""

# Add our bacterium and our food source to the env
env.add_agent(bm.Bacterium(name="Rick", goal="work"))

env.add_agent(bm.FoodSource(name="Runaway Food", goal="not get eaten"))

utils.run_model(env, prog_file, results_file)



"""
timer_model.py
An agent that does nothing to allow timing of other code portions
more accurately.
"""
import indra.entity as ent


class TimerAgent1(ent.Agent):
    """
    An agent that just prints that it is around when asked to act
    """

    def __init__(self, name, goal):
        """
        A very basic init.
        """
        super().__init__(name, goal)


    def act(self):
        """
        Just pass.
        """
        pass


class TimerAgent2(ent.Agent):
    """
    An agent that just prints that it is around when asked to act
    """

    def __init__(self, name, goal):
        """
        A very basic init.
        """
        super().__init__(name, goal)


    def act(self):
        """
        Just pass.
        """
        pass


class TimerAgent3(ent.Agent):
    """
    An agent that just prints that it is around when asked to act
    """

    def __init__(self, name, goal):
        """
        A very basic init.
        """
        super().__init__(name, goal)


    def act(self):
        """
        Just pass.
        """
        pass


class TimerEnv(ent.Environment):
    """
    This environment doesn't really do anything.
    """

    def __init__(self, model_nm=None):
        super().__init__("Timer environment",
                         model_nm=model_nm)
 

    def act_loop(self):
        """
        We can change the looping behavior here to
        test the efficiency of various looping methods.
        """
        for agent in reversed(self.agents):
            agent.act()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 18 16:45:50 2014

@author: Gene Callahan and Brandon Logan
"""

import indra.utils as utils
import indra.prop_args as props
import height_model as hm

START_HEIGHT = 100.0

MODEL_NM = "height_model"
(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_agent_num(pa, "num_agents", "agents", 80)

env = hm.HeightEnv(model_nm=MODEL_NM)

for i in range(pa.get("num_agents")):
        env.add_agent(
            hm.HeightAgentEng('Eng agent' + str(i),
                              START_HEIGHT, START_HEIGHT))
        env.add_agent(
            hm.HeightAgent('agent' + str(i), START_HEIGHT, START_HEIGHT))

utils.run_model(env, prog_file, results_file)



from setuptools import setup, find_packages

setup(name='indras_net',
    version='1.1',
    description='A framework for agent-based modeling in Python.',
    url='http://github.com/gcallah/indra',
    author='Gene Callahan and Nathan Conroy',
    author_email='gcallah@mac.com',
    license='GNU',
    zip_safe=False,
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*",
                                 "tests"]),
    install_requires=[
          "networkx",
          "numpy",
    ],
    test_suite="",
    entry_points={
        "console_scripts": ["bigbox = bigbox.__main__:main",
                            "schelling = schelling.__main__:main",
                            "models = models.__main__:main"
                           ]
    },
)



"""
A script that runs big_box_model. It simulates the market economy
of consumers, mom and pops, and big boxes.
"""

import indra.utils as utils
import indra.prop_args as props
import big_box_model as bb

# set up some file names:
MODEL_NM = "BigBoxModel"

(prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)

# We store basic parameters in a "property" file; this allows us to save
#  multiple parameter sets, which is important in simulation work.
#  We can read these in from file or set them here.
pa = utils.read_props(MODEL_NM)
if pa is None:
    pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
    utils.get_grid_dims(pa, 50)
    utils.get_agent_num(pa, "num_consumers", "consumers",
                        bb.NUM_GOODS * 5 + 1)
    utils.get_agent_num(pa, "num_mom_and_pops", "mom and pop stores",
                        bb.NUM_GOODS)
    pa.ask("allowance", "What are the consumers' daily allowances?", 
           int, default=2)
# right now, big box gets a multiple of m&p expenses and endowment
    pa.ask("endowment", "What are the small shops' initial endowments?", 
           int, default=30)
    pa.ask("expenses", "What are the small shops' expenses per period?",
           int, default=10)
    pa.ask("bb_start_period", "At what period should big boxes appear?",
           int, default=20, limits=(1, 100))
    pa.ask("pref_for_mp",
           "What's the consumer preference for mom and pops?",
           float, default=0.2, limits=(0.0, 1.0))

# Now we create a town for our agents to act in:
env = bb.EverytownUSA(pa.get("grid_width"),
                      pa.get("grid_height"),
                      model_nm=MODEL_NM)

# Now we loop creating multiple agents with numbered names
# based on the number of agents of that type to create:
for i in range(pa.get("num_consumers")):
    env.add_agent(bb.Consumer("consumer" + str(i),
                              goal=(i % bb.NUM_GOODS),
                              init_state=0,
                              allowance=pa.get("allowance")))
for i in range(pa.get("num_mom_and_pops")):
    env.add_agent(bb.MomAndPop("mom_and_pop" + str(i),
                               goal=(i % bb.NUM_GOODS),
                               endowment=pa.get("endowment"),
                               expenses=pa.get("expenses"),
                               adj=pa.get("pref_for_mp")
                               ))

utils.run_model(env, prog_file, results_file)



from indra.entry_point import entry_point

models = {
    "1": ("Segregation", "segregation_run.py"),
    "2": ("Height", "height_run.py"),
    "3": ("Auditorium", "auditorium_run.py")
     }


def main(args=None):
    entry_point(args, models)

if __name__ == "__main__":
    main()
